@@ -37,9 +37,9 @@ public:
     std::string getCurrent() override;
     std::string getDefault() override;
     void deleteSnap(std::string id) override;
+    void rollbackTo(std::string id) override;
 private:
     std::string callSnapper(std::string);
-    inline static bool snapperNoDbus;
 };
 
 } // namespace TransactionalUpdate
